@@ -1,27 +1,29 @@
 /*
- * This file is part of oFono - Open Source Telephony
  *
- * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
+ *  oFono - Open Source Telephony
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
 #ifndef __ISIMODEM_DEBUG_H
 #define __ISIMODEM_DEBUG_H
 
+#include <gisi/message.h>
+
 #include "ss.h"
 #include "mtc.h"
 #include "sms.h"
@@ -39,6 +41,14 @@ const char *ss_ussd_type_name(enum ss_ussd_type value);
 const char *mtc_isi_cause_name(enum mtc_isi_cause value);
 const char *mtc_message_id_name(enum mtc_message_id value);
 const char *mtc_modem_state_name(enum mtc_modem_state value);
+const char *mce_rf_state_name(enum mce_rf_state value);
+
+const char *mce_message_id_name(enum mce_message_id value);
+const char *mce_modem_state_name(enum mce_modem_state value);
+const char *mce_status_info(enum mce_status_info value);
+
+const char *uicc_subblock_name(uint8_t value);
+const char *uicc_status_name(uint8_t value);
 
 const char *sms_gsm_cause_name(enum sms_gsm_cause value);
 const char *sms_isi_cause_name(enum sms_isi_cause value);
@@ -74,15 +84,7 @@ const char *gpds_isi_cause_name(enum gpds_isi_cause value);
 const char *gpds_transfer_status_name(enum gpds_transfer_status value);
 const char *gpds_transfer_cause_name(enum gpds_transfer_cause value);
 
-void ss_debug(const void *restrict buf, size_t len, void *data);
-void mtc_debug(const void *restrict buf, size_t len, void *data);
-void sms_debug(const void *restrict buf, size_t len, void *data);
-void sim_debug(const void *restrict buf, size_t len, void *data);
-void info_debug(const void *restrict buf, size_t len, void *data);
-void call_debug(const void *restrict buf, size_t len, void *data);
-void net_debug(const void *restrict buf, size_t len, void *data);
-void gss_debug(const void *restrict buf, size_t len, void *data);
-void gpds_debug(const void *restrict buf, size_t len, void *data);
+void isi_trace(const GIsiMessage *msg, void *data);
 
 const char *pn_resource_name(int value);
 
